#!/usr/bin/python
# -*- coding: utf-8 -*-

# Libervia: a Salut à Toi frontend
# Copyright (C) 2011, 2012, 2013, 2014, 2015 Jérôme Poisson <goffi@goffi.org>
# Copyright (C) 2011, 2012  Adrien Vigneron <adrienvigneron@mailoo.org>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#This page manage subscription and new account creation

import pyjd  # this is dummy in pyjs
from sat.core.i18n import _

from pyjamas.ui.SimplePanel import SimplePanel
from pyjamas.ui.VerticalPanel import VerticalPanel
from pyjamas.ui.HorizontalPanel import HorizontalPanel
from pyjamas.ui.StackPanel import StackPanel
from pyjamas.ui.PasswordTextBox import PasswordTextBox
from pyjamas.ui.TextBox import TextBox
from pyjamas.ui.FormPanel import FormPanel
from pyjamas.ui.Button import Button
from pyjamas.ui.Label import Label
from pyjamas.ui.HTML import HTML
from pyjamas.ui.PopupPanel import PopupPanel
from pyjamas.ui.Image import Image
from pyjamas.ui.Hidden import Hidden
from pyjamas import Window
from pyjamas.ui.KeyboardListener import KEY_ENTER
from pyjamas.Timer import Timer

import re

from constants import Const as C


class RegisterPanel(FormPanel):

    def __init__(self, callback):
        """
        @param callback: method to call if login successful
        """
        FormPanel.__init__(self)
        self.setSize('600px', '350px')
        self.callback = callback
        self.setMethod(FormPanel.METHOD_POST)
        main_panel = HorizontalPanel()
        main_panel.setStyleName('registerPanel_main')
        left_side = Image("media/libervia/register_left.png")
        main_panel.add(left_side)

        ##StackPanel##
        self.right_side = StackPanel(StyleName='registerPanel_right_side')
        main_panel.add(self.right_side)
        main_panel.setCellWidth(self.right_side, '100%')

        ##Login stack##
        login_stack = SimplePanel()
        login_stack.setStyleName('registerPanel_content')
        login_vpanel = VerticalPanel()
        login_stack.setWidget(login_vpanel)

        self.login_warning_msg = HTML('')
        self.login_warning_msg.setStyleName('formWarning')
        login_vpanel.add(self.login_warning_msg)

        login_label = Label('Login:')
        self.login_box = TextBox()
        self.login_box.setName("login")
        self.login_box.addKeyboardListener(self)
        login_pass_label = Label('Password:')
        self.login_pass_box = PasswordTextBox()
        self.login_pass_box.setName("login_password")
        self.login_pass_box.addKeyboardListener(self)

        login_vpanel.add(login_label)
        login_vpanel.add(self.login_box)
        login_vpanel.add(login_pass_label)
        login_vpanel.add(self.login_pass_box)
        login_but = Button("Log in", getattr(self, "onLogin"))
        login_but.setStyleName('button')
        login_but.addStyleName('red')
        login_vpanel.add(login_but)

        #The hidden submit_type field
        self.submit_type = Hidden('submit_type')
        login_vpanel.add(self.submit_type)

        ##Register stack##
        register_stack = SimplePanel()
        register_stack.setStyleName('registerPanel_content')
        register_vpanel = VerticalPanel()
        register_stack.setWidget(register_vpanel)

        self.register_warning_msg = HTML('')
        self.register_warning_msg.setStyleName('formWarning')
        register_vpanel.add(self.register_warning_msg)

        register_login_label = Label('Login:')
        self.register_login_box = TextBox()
        self.register_login_box.setName("register_login")
        self.register_login_box.addKeyboardListener(self)
        email_label = Label('E-mail:')
        self.email_box = TextBox()
        self.email_box.setName("email")
        self.email_box.addKeyboardListener(self)
        register_pass_label = Label('Password:')
        self.register_pass_box = PasswordTextBox()
        self.register_pass_box.setName("register_password")
        self.register_pass_box.addKeyboardListener(self)
        register_vpanel.add(register_login_label)
        register_vpanel.add(self.register_login_box)
        register_vpanel.add(email_label)
        register_vpanel.add(self.email_box)
        register_vpanel.add(register_pass_label)
        register_vpanel.add(self.register_pass_box)

        register_but = Button("Register a new account", getattr(self, "onRegister"))
        register_but.setStyleName('button')
        register_but.addStyleName('red')
        register_vpanel.add(register_but)

        self.right_side.add(login_stack, 'Return to the login screen')
        self.right_side.add(register_stack, 'No account yet? Create a new one!')
        self.right_side.addStackChangeListener(self)
        login_stack.setWidth(None)
        register_stack.setWidth(None)

        self.add(main_panel)
        self.addFormHandler(self)
        self.setAction('register_api/login')

    def onStackChanged(self, sender, index):
        if index == 0:
            self.login_box.setFocus(True)
        elif index == 1:
            self.register_login_box.setFocus(True)

    def onKeyPress(self, sender, keycode, modifiers):
        if keycode == KEY_ENTER:
            # Browsers offer an auto-completion feature to any
            # text box, but the selected value is not set when
            # the widget looses the focus. Using a timer with
            # any delay value > 0 would do the trick.
            if sender == self.login_box:
                Timer(5, lambda timer: self.login_pass_box.setFocus(True))
            elif sender == self.login_pass_box:
                self.onLogin(None)
            elif sender == self.register_login_box:
                Timer(5, lambda timer: self.email_box.setFocus(True))
            elif sender == self.email_box:
                Timer(5, lambda timer: self.register_pass_box.setFocus(True))
            elif sender == self.register_pass_box:
                self.onRegister(None)

    def onKeyUp(self, sender, keycode, modifiers):
        pass

    def onKeyDown(self, sender, keycode, modifiers):
        pass

    def onLogin(self, button):
        if not re.match(r'^[a-z0-9_-]+(@[a-z0-9_-]+\.[a-z0-9_-]+)?$', self.login_box.getText(), re.IGNORECASE):
            self.login_warning_msg.setHTML('Invalid login, valid characters<br>are a-z A-Z 0-9 _ - or a bare JID')
        else:
            self.submit_type.setValue('login')
            self.submit(None)

    def onRegister(self, button):
        # XXX: for now libervia forces the creation to lower case
        self.register_login_box.setText(self.register_login_box.getText().lower())
        if not re.match(r'^[a-z0-9_-]+$', self.register_login_box.getText(), re.IGNORECASE):
            self.register_warning_msg.setHTML(_('Invalid login, valid characters<br>are a-z A-Z 0-9 _ -'))
        elif not re.match(r'^[a-z0-9_-]+@[a-z0-9_-]+\.[a-z0-9_-]+$', self.email_box.getText(), re.IGNORECASE):
            self.register_warning_msg.setHTML(_('Invalid email address'))
        elif len(self.register_pass_box.getText()) < C.PASSWORD_MIN_LENGTH:
            self.register_warning_msg.setHTML(_('Your password must contain<br>at least %d characters.') % C.PASSWORD_MIN_LENGTH)
        else:
            self.register_warning_msg.setHTML("")
            self.submit_type.setValue('register')
            self.submit(None)

    def onSubmit(self, event):
        pass

    def onSubmitComplete(self, event):
        result = event.getResults()
        if result == C.PROFILE_AUTH_ERROR:
            self.login_warning_msg.setHTML(_('Your login and/or password is incorrect. Please try again.'))
        elif result == C.XMPP_AUTH_ERROR:
            # TODO: call stdui action CHANGE_XMPP_PASSWD_ID as it's done in primitivus
            Window.alert(_(u'Your SàT profile has been authenticated but the associated XMPP account failed to connect. Please use another SàT frontend to set another XMPP password.'))
        elif result == C.PROFILE_LOGGED:
            self.callback()
        elif result == C.SESSION_ACTIVE:
            Window.alert(_('Session already active, this should not happen, please contact the author to fix it.'))
        elif result == C.NO_REPLY:
            Window.alert(_("Did not receive a reply (the timeout expired or the connection is broken)."))
        elif result == C.ALREADY_EXISTS:
            self.register_warning_msg.setHTML(_('This login already exists,<br>please choose another one.'))
        elif result == C.INTERNAL_ERROR:
            self.register_warning_msg.setHTML(_('An registration error occurred, please contact the server administrator.'))
        elif result == C.REGISTRATION_SUCCEED:
            self.login_warning_msg.setHTML("")
            self.register_warning_msg.setHTML("")
            self.login_box.setText(self.register_login_box.getText())
            self.login_pass_box.setText('')
            self.register_login_box.setText('')
            self.register_pass_box.setText('')
            self.email_box.setText('')
            self.right_side.showStack(0)
            self.login_pass_box.setFocus(True)
            Window.alert(_('An email has been sent to you with your login informations\nPlease remember that this is ONLY A TECHNICAL DEMO.'))
        else:
            Window.alert(_('Submit error: %s' % result))


class RegisterBox(PopupPanel):

    def __init__(self, callback, *args, **kwargs):
        PopupPanel.__init__(self, *args, **kwargs)
        self._form = RegisterPanel(callback)
        self.setWidget(self._form)

    def onWindowResized(self, width, height):
        super(RegisterBox, self).onWindowResized(width, height)
        self.centerBox()

    def show(self):
        super(RegisterBox, self).show()
        self.centerBox()
        self._form.login_box.setFocus(True)
